package com.interswitchng.isw_mobile_sdk;

import android.app.Application;

import androidx.annotation.NonNull;

import com.interswitchng.iswmobilesdk.IswMobileSdk;
import com.interswitchng.iswmobilesdk.shared.models.core.Environment;
import com.interswitchng.iswmobilesdk.shared.models.core.IswPaymentInfo;
import com.interswitchng.iswmobilesdk.shared.models.core.IswPaymentResult;
import com.interswitchng.iswmobilesdk.shared.models.core.IswSdkConfig;

import java.util.HashMap;
import java.util.Map;

import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;

/** IswMobileSdkPlugin */
public class IswMobileSdkPlugin implements FlutterPlugin, MethodCallHandler {
  /// The MethodChannel that will the communication between Flutter and native Android
  ///
  /// This local reference serves to register the plugin with the Flutter Engine and unregister it
  /// when the Flutter Engine is detached from the Activity
  private MethodChannel channel;
  private Application application;

  @Override
  public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
    application = (Application) flutterPluginBinding.getApplicationContext();
    channel = new MethodChannel(
            flutterPluginBinding.getBinaryMessenger(),
            "com.interswitchng.isw_mobile_sdk"
    );
    channel.setMethodCallHandler(this);
  }

  @Override
  public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
    switch (call.method) {
      case "initialize":
        Map<String, String> configMap = call.argument("config");
        String env = call.argument("env");
        assert configMap != null;
        assert env != null;

        IswMobileSdk.initialize(application, getConfig(configMap, env));
        result.success(null);
        break;
      case "pay":
        Map<String, String> infoMap = call.argument("paymentInfo");
        assert infoMap != null;
        IswMobileSdk.getInstance().pay(getPaymentInfo(infoMap), new IswSdkCallback(call, result));
        break;
      default:
        result.notImplemented();
    }
  }

  @Override
  public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
    application = null;
    channel.setMethodCallHandler(null);
  }


  private IswSdkConfig getConfig(Map<String, String> configMap, String env) {
    // extract configurations
    String merchantId = configMap.get("merchantId"),
            merchantSecret = configMap.get("merchantSecret"),
            merchantCode = configMap.get("merchantCode"),
            currencyCode = configMap.get("currencyCode");

    boolean isProduction = env.equals(Environment.PRODUCTION.name());
    boolean isSandbox = env.equals(Environment.SANDBOX.name());

    // create configuration
    IswSdkConfig config = new IswSdkConfig(merchantId, merchantSecret, merchantCode, currencyCode);

    // set environment
    if (isProduction) config.setEnv(Environment.PRODUCTION);
    else if (isSandbox) config.setEnv(Environment.SANDBOX);


    return config;
  }

  private IswPaymentInfo getPaymentInfo(Map<String, String> infoMap) {

    String customerId = infoMap.get("customerId"),
            customerName = infoMap.get("customerName"),
            customerEmail = infoMap.get("customerEmail"),
            customerMobile = infoMap.get("customerMobile"),
            reference = infoMap.get("reference"),
            amount = infoMap.get("amount");

    return new IswPaymentInfo(customerId,
            customerName,
            customerEmail,
            customerMobile,
            reference,
            Integer.valueOf(amount)
    );
  }

  static class IswSdkCallback implements IswMobileSdk.IswPaymentCallback {
    MethodCall call;
    Result result;

    public IswSdkCallback(MethodCall call, Result result) {
      this.call = call;
      this.result = result;
    }


    @Override
    public void onPaymentCompleted(@NonNull IswPaymentResult iswPaymentResult) {
      this.result.success(getResult(iswPaymentResult));
    }

    @Override
    public void onUserCancel() {
      this.result.success(null);
    }

    private Map<String, String> getResult(IswPaymentResult result) {
      Map<String, String> map = new HashMap<>();
      map.put("responseCode", result.getResponseCode());
      map.put("responseDescription", result.getResponseDescription());
      map.put("transactionReference", result.getTransactionReference());
      map.put("amount", result.getAmount() + "");
      map.put("isSuccessful", result.isSuccessful() + "");
      map.put("channel", result.getChannel().name());
      return map;
    }
  }
}
