import 'package:flutter/services.dart';

import 'models/isw_mobile_sdk_payment_info.dart';
import 'models/isw_mobile_sdk_payment_result.dart';
import 'models/isw_mobile_sdk_sdk_config.dart';

class IswMobileSdk {
  static const _channel = MethodChannel('com.interswitchng.isw_mobile_sdk');

  static Future<void> initialize(
    IswSdkConfig config,
    [Environment env = Environment.TEST]
  ) async {
    // Errors occurring on the platform side cause invokeMethod to throw
    // PlatformExceptions.
    try {
      return _channel.invokeMethod('initialize', {'config': config.toMap(), 'env': nameOf(env)});
    } on PlatformException {
      throw 'Unable to configure sdk';
    }
  }

  static Future<Optional<IswPaymentResult?>> pay(IswPaymentInfo info) async {
    try {
      final result =
          await _channel.invokeMethod('pay', {'paymentInfo': info.toMap()});
      if (result == null) {
        return Optional(null, false);
      } else {
        return Optional(IswPaymentResult.fromJson(result), true);
      }
    } on PlatformException {
      throw 'Unable to complete payment';
    }
  }

  static String nameOf(Environment env) {
    final name = env.toString();
    return name.substring(name.indexOf('.') + 1);
  }
}